{{
   demo operation of the D6t thermal imaging sensor, 16 pixels 4x4
   Tracy Allen, EME Systems,
   (c) 2014, MIT license, see end of program for terms of use
   Note operation at 10MHz for this sensor, which is slow.
   }}


CON
        _clkmode = xtal1 + pll2x     'Standard clock mode * crystal frequency = 10 MHz
        _xinfreq = 5_000_000

  SDA = 1
  SCL = 0

  D6T_ID = $14  ' 8 bit write address
  D6T_CONVERT = $4c

   
OBJ
  uarts       : "fullDuplexSerial4port"   'Debug terminal for communication
  i2c    : "i2c_1bus_stretch"

VAR
   byte array[36]
  
PUB main | ack, idx, crc
  startUarts
  waitcnt(clkfreq/10+cnt)

  uarts.str(0,string(13,"demo of D6T, reading temperatures."))
  if ina[SDA] & ina[SCL] == 0
    uarts.str(0,string(13,"missing pullup"))
    repeat
  if i2c.Init(SCL,SDA) == 1
    uarts.str(0,string(13,"bad i2c"))
    repeat
  uarts.str(0,string(13,"device found"))
  repeat
    waitcnt(clkfreq+cnt)
    ack := i2c.readPage(D6T_ID, D6T_CONVERT, @array, 35)
      uarts.tx(0,13)
      repeat idx from 0 to 34
        uarts.hex(0,array[idx],2)
        uarts.char(0,32)
    crc := Crc8Calc(@array, 34, $6b)
    uarts.tx(0,13)
    uarts.hex(0,array[34],2)
    uarts.hex(0,crc,2)

PUB startUarts
  uarts.init
  uarts.addport(0,31,30,-1,-1,0,0,9600)
  uarts.start

PUB ScanI2Cbus | idx, flag
  uarts.tx(0,13)
  flag~
  if ina[SDA] & ina[SCL] == 0
    uarts.str(0,string("missing pullup",13))
    repeat
  repeat idx from 0 to $fe step 2
    if i2c.Hello(idx) == 0
      flag++
      uarts.str(0,string(13,"i2c address found: "))
      uarts.hex(0,idx,2)
  if flag
    uarts.str(0,string(13,"total found="))
    uarts.dec(0,flag)
  else
    uarts.str(0,string(13,"no i2c devices found"))
  uarts.tx(0,13)
  repeat

PUB Crc8Calc(dataPointer, dataCount, crc)
' enter with data pointer to data bytes
  uarts.tx(0,13)
  Repeat dataCount
    crc := Crc8shift(byte[datapointer++] ^ crc)
    uarts.hex(0,crc,2)
    uarts.char(0,32)
  return crc

PUB crc8shift(datum) | temp
  repeat 8
    datum <<= 1
    if datum & $100
        datum ^= $07
  return datum & $ff



{{
┌─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                 │                                                            
├─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation   │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,   │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the        │
│Software is furnished to do so, subject to the following conditions:                                                         │         
│                                                                                                                             │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the         │
│Software.                                                                                                                    │
│                                                                                                                             │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR        │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,  │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                        │
└─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}  
